@@ -11,16 +11,22 @@
 #define KICONLOADER_H
 
 #include <QObject>
+#include <QSize>
 #include <QString>
 #include <QStringList>
 #include <memory>
 
+#if __has_include(<optional>) && __cplusplus >= 201703L
+#include <optional>
+#endif
+
 #include <kiconthemes_export.h>
 
 class QIcon;
 class QMovie;
 class QPixmap;
 
+class KIconColors;
 class KIconLoaderPrivate;
 class KIconEffect;
 class KIconTheme;
@@ -78,7 +84,8 @@ public:
         Application, ///< An icon that represents an application.
         Device, ///< An icon that represents a device.
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(4, 8)
-        FileSystem, ///< An icon that represents a file system. @deprecated Since 4.8. Use Place instead.
+        FileSystem ///< An icon that represents a file system. @deprecated Since 4.8. Use Place instead.
+            KICONTHEMES_ENUMERATOR_DEPRECATED_VERSION_BELATED(5, 82, 4, 8, "Use Place instead"),
 #elif KICONTHEMES_BUILD_DEPRECATED_SINCE(4, 8)
         FileSystem_DEPRECATED_DO_NOT_USE,
 #endif
@@ -129,6 +136,7 @@ public:
         /// Small icons, e.g. for buttons
         Small,
         /// Panel (Plasma Taskbar) icons
+        // TODO KF6: remove this (See https://phabricator.kde.org/T14340)
         Panel,
         /// Icons for use in dialog titles, page lists, etc
         Dialog,
@@ -186,7 +194,7 @@ public:
     /**
      * Cleanup
      */
-    ~KIconLoader();
+    ~KIconLoader() override;
 
     /**
      * Returns the global icon loader initialized with the application name.
@@ -282,6 +290,97 @@ public:
                            QString *path_store = nullptr,
                            bool canReturnNull = false) const;
 
+    /**
+     * Loads an icon. It will try very hard to find an icon which is
+     * suitable. If no exact match is found, a close match is searched.
+     * If neither an exact nor a close match is found, a null pixmap or
+     * the "unknown" pixmap is returned, depending on the value of the
+     * @p canReturnNull parameter.
+     *
+     * @param name The name of the icon, without extension.
+     * @param group The icon group. This will specify the size of and effects to
+     * be applied to the icon.
+     * @param scale The scale of the icon group to use. If no icon exists in the
+     * scaled group, a 1x icon with its size multiplied by the scale will be
+     * loaded instead.
+     * @param size If nonzero, this overrides the size specified by @p group.
+     *             See KIconLoader::StdSizes. The icon will be fit into @p size
+     *             without changing the aspect ratio, which particularly matters
+     *             for non-square icons.
+     * @param state The icon state: @p DefaultState, @p ActiveState or
+     * @p DisabledState. Depending on the user's preferences, the iconloader
+     * may apply a visual effect to hint about its state.
+     * @param overlays a list of emblem icons to overlay, by name
+     *                 @see drawOverlays
+     * @param path_store If not null, the path of the icon is stored here,
+     *        if the icon was found. If the icon was not found @p path_store
+     *        is unaltered even if the "unknown" pixmap was returned.
+     * @param canReturnNull Can return a null pixmap? If false, the
+     *        "unknown" pixmap is returned when no appropriate icon has been
+     *        found. <em>Note:</em> a null pixmap can still be returned in the
+     *        event of invalid parameters, such as empty names, negative sizes,
+     *        and etc.
+     * @return the QPixmap. Can be null when not found, depending on
+     *         @p canReturnNull.
+     * @since 5.81
+     */
+    QPixmap loadScaledIcon(const QString &name,
+                           KIconLoader::Group group,
+                           qreal scale,
+                           const QSize &size = {},
+                           int state = KIconLoader::DefaultState,
+                           const QStringList &overlays = QStringList(),
+                           QString *path_store = nullptr,
+                           bool canReturnNull = false) const;
+
+#if __has_include(<optional>) && __cplusplus >= 201703L
+    /**
+     * Loads an icon. It will try very hard to find an icon which is
+     * suitable. If no exact match is found, a close match is searched.
+     * If neither an exact nor a close match is found, a null pixmap or
+     * the "unknown" pixmap is returned, depending on the value of the
+     * @p canReturnNull parameter.
+     *
+     * @param name The name of the icon, without extension.
+     * @param group The icon group. This will specify the size of and effects to
+     * be applied to the icon.
+     * @param scale The scale of the icon group to use. If no icon exists in the
+     * scaled group, a 1x icon with its size multiplied by the scale will be
+     * loaded instead.
+     * @param size If nonzero, this overrides the size specified by @p group.
+     *             See KIconLoader::StdSizes. The icon will be fit into @p size
+     *             without changing the aspect ratio, which particularly matters
+     *             for non-square icons.
+     * @param state The icon state: @p DefaultState, @p ActiveState or
+     * @p DisabledState. Depending on the user's preferences, the iconloader
+     * may apply a visual effect to hint about its state.
+     * @param overlays a list of emblem icons to overlay, by name
+     *                 @see drawOverlays
+     * @param path_store If not null, the path of the icon is stored here,
+     *        if the icon was found. If the icon was not found @p path_store
+     *        is unaltered even if the "unknown" pixmap was returned.
+     * @param canReturnNull Can return a null pixmap? If false, the
+     *        "unknown" pixmap is returned when no appropriate icon has been
+     *        found. <em>Note:</em> a null pixmap can still be returned in the
+     *        event of invalid parameters, such as empty names, negative sizes,
+     *        and etc.
+     * @param colorScheme will define the stylesheet used to color this icon.
+     *        Note this will only work if @p name is an svg file.
+     * @return the QPixmap. Can be null when not found, depending on
+     *         @p canReturnNull.
+     * @since 5.88
+     */
+    QPixmap loadScaledIcon(const QString &name,
+                           KIconLoader::Group group,
+                           qreal scale,
+                           const QSize &size,
+                           int state,
+                           const QStringList &overlays,
+                           QString *path_store,
+                           bool canReturnNull,
+                           const std::optional<KIconColors> &colorScheme) const;
+#endif
+
     /**
      * Loads an icon for a mimetype.
      * This is basically like loadIcon except that extra desktop themes are loaded if necessary.
@@ -497,14 +596,17 @@ public:
      */
     static QPixmap unknown();
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 82)
     /**
      * Checks whether the user wants to blend the icons with the background
      *  using the alpha channel information for a given group.
      * @param group the group to check
      * @return true if alpha blending is desired
-     * @obsolete
+     * @deprecated Since 3.1, no known users
      */
+    KICONTHEMES_DEPRECATED_VERSION_BELATED(5, 82, 3, 1, "No known users")
     bool alphaBlending(KIconLoader::Group group) const;
+#endif
 
     /**
      * Draws overlays on the specified pixmap, it takes the width and height
@@ -547,6 +649,14 @@ public:
      */
     void resetPalette();
 
+    /**
+     * @returns whether we have set a custom palette using @f setCustomPalette
+     *
+     * @since 5.85
+     * @see resetPalette, setCustomPalette
+     */
+    bool hasCustomPalette() const;
+
 public Q_SLOTS:
     // TODO: while marked as deprecated, newIconLoader() is still used:
     // internally by KIconLoadeer as well as by Plasma's Icons kcm module (state: 5.17)
@@ -585,6 +695,7 @@ Q_SIGNALS:
     void iconChanged(int group);
 
 private:
+    friend class KIconLoaderPrivate;
     // @internal the data object
     std::unique_ptr<KIconLoaderPrivate> const d;
 
@@ -599,8 +710,9 @@ private:
  * if you need a pixmap use QIcon::pixmap with KIconLoader::StdSizes,
  * if you need the overlay, use KIconLoader::loadIcon.
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 63, "See API dox for replacement")
-KICONTHEMES_EXPORT QPixmap DesktopIcon(const QString &name, int size = 0, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
+QPixmap DesktopIcon(const QString &name, int size = 0, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
 #endif
 
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
@@ -609,8 +721,9 @@ KICONTHEMES_EXPORT QPixmap DesktopIcon(const QString &name, int size = 0, int st
  * Load a desktop icon, and apply the necessary effects to get an IconSet.
  * @deprecated Since 5.0, use QIcon::fromTheme instead
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
-KICONTHEMES_EXPORT QIcon DesktopIconSet(const QString &name, int size = 0);
+QIcon DesktopIconSet(const QString &name, int size = 0);
 #endif
 
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 63)
@@ -621,8 +734,9 @@ KICONTHEMES_EXPORT QIcon DesktopIconSet(const QString &name, int size = 0);
  * if you need a pixmap use QIcon::pixmap with KIconLoader::StdSizes,
  * if you need the overlay, use KIconLoader::loadIcon.
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 63, "See API dox for replacement")
-KICONTHEMES_EXPORT QPixmap BarIcon(const QString &name, int size = 0, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
+QPixmap BarIcon(const QString &name, int size = 0, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
 #endif
 
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
@@ -631,8 +745,9 @@ KICONTHEMES_EXPORT QPixmap BarIcon(const QString &name, int size = 0, int state
  * Load a toolbar icon, and apply the necessary effects to get an IconSet.
  * @deprecated Since 5.0, use QIcon::fromTheme instead
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
-KICONTHEMES_EXPORT QIcon BarIconSet(const QString &name, int size = 0);
+QIcon BarIconSet(const QString &name, int size = 0);
 #endif
 
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 63)
@@ -643,8 +758,9 @@ KICONTHEMES_EXPORT QIcon BarIconSet(const QString &name, int size = 0);
  * if you need a pixmap use QIcon::pixmap with KIconLoader::StdSizes,
  * if you need the overlay, use KIconLoader::loadIcon.
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 63, "See API dox for replacement")
-KICONTHEMES_EXPORT QPixmap SmallIcon(const QString &name, int size = 0, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
+QPixmap SmallIcon(const QString &name, int size = 0, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
 #endif
 
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
@@ -653,8 +769,9 @@ KICONTHEMES_EXPORT QPixmap SmallIcon(const QString &name, int size = 0, int stat
  * Load a small icon, and apply the necessary effects to get an IconSet.
  * @deprecated Since 5.0, use QIcon::fromTheme instead
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
-KICONTHEMES_EXPORT QIcon SmallIconSet(const QString &name, int size = 0);
+QIcon SmallIconSet(const QString &name, int size = 0);
 #endif
 
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 63)
@@ -665,8 +782,9 @@ KICONTHEMES_EXPORT QIcon SmallIconSet(const QString &name, int size = 0);
  * if you need a pixmap use QIcon::pixmap with KIconLoader::StdSizes,
  * if you need the overlay, use KIconLoader::loadIcon.
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 63, "See API dox for replacement")
-KICONTHEMES_EXPORT QPixmap MainBarIcon(const QString &name, int size = 0, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
+QPixmap MainBarIcon(const QString &name, int size = 0, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
 #endif
 
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
@@ -675,8 +793,9 @@ KICONTHEMES_EXPORT QPixmap MainBarIcon(const QString &name, int size = 0, int st
  * Load a main toolbar icon, and apply the effects to get an IconSet.
  * @deprecated Since 5.0, use QIcon::fromTheme instead
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
-KICONTHEMES_EXPORT QIcon MainBarIconSet(const QString &name, int size = 0);
+QIcon MainBarIconSet(const QString &name, int size = 0);
 #endif
 
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 65)
@@ -686,8 +805,9 @@ KICONTHEMES_EXPORT QIcon MainBarIconSet(const QString &name, int size = 0);
  * @deprecated since 5.65 Prefer qrc files over user icon themes.
  * If that's not an option for now, use KIconLoader::loadIcon.
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 65, "See API dox for replacement")
-KICONTHEMES_EXPORT QPixmap UserIcon(const QString &name, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
+QPixmap UserIcon(const QString &name, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
 #endif
 
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
@@ -696,8 +816,9 @@ KICONTHEMES_EXPORT QPixmap UserIcon(const QString &name, int state = KIconLoader
  * Load a user icon, and apply the effects to get an IconSet.
  * @deprecated Since 5.0, use QIcon::fromTheme instead
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
-KICONTHEMES_EXPORT QIcon UserIconSet(const QString &name);
+QIcon UserIconSet(const QString &name);
 #endif
 
 #if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 66)
@@ -707,8 +828,9 @@ KICONTHEMES_EXPORT QIcon UserIconSet(const QString &name);
  * @deprecated since 5.66 Prefer QStyle::pixelMetric.
  * If that's not an option for now, use KIconLoader::currentSize.
  */
+KICONTHEMES_EXPORT
 KICONTHEMES_DEPRECATED_VERSION(5, 66, "Use QStyle::pixelMetric or KIconLoader::currentSize")
-KICONTHEMES_EXPORT int IconSize(KIconLoader::Group group);
+int IconSize(KIconLoader::Group group);
 #endif
 
 namespace KDE
@@ -721,6 +843,7 @@ namespace KDE
  * @since 5.0
  */
 KICONTHEMES_EXPORT QIcon icon(const QString &iconName, KIconLoader *iconLoader = nullptr);
+KICONTHEMES_EXPORT QIcon icon(const QString &iconName, const KIconColors &colors, KIconLoader *iconLoader = nullptr);
 
 /**
  * \relates KIconLoader
